﻿using System;
using System.Runtime.Serialization;

namespace Facebook
{
    /// <summary>Defines a contract for an object that contains basic information about an authenticated Facebook session.</summary>
    public interface ISessionInfo : ISerializable
    {
        /// <summary>Gets or sets the session key generated by the Facebook API for the current session.</summary>
        String SessionKey { get; set; }

        /// <summary>Gets or sets the id of the Facebook user.</summary>
        Int64 Uid { get; set; }

        /// <summary>Gets or sets a <see cref="DateTime" /> value representing when the session expires.</summary>
        DateTime Expires { get; set; }

        /// <summary>Gets or sets the session secret used for communicating with the API with JavaScript or Flash.</summary>
        /// <remarks>See Facebook's documentation on <a href="http://wiki.developers.facebook.com/index.php/Session_Secret">Session Secret</a>.</remarks>
        String Secret { get; set; }

        /// <summary>Gets or sets the base domain for web applications when the application is run on multiple domains.</summary>
        /// <remarks>See Facebook's documentation on <a href="http://wiki.developers.facebook.com/index.php/Base_Domain">Base Domain</a>.</remarks>
        String BaseDomain { get; set; }

        /// <summary>Gets or sets the API key of the application the session was created for.</summary>
        String ApiKey { get; set; }

        /// <summary>In implementing classes ,this method should initialize the <see cref="ISessionInfo" /> instance, using the specified <see cref="ISessionInfo" /> object to populate its properties.</summary>
        /// <param name="sessionInfo">A <see cref="ISessionInfo" /> object used to overwrite the data in the current instance.</param>
        void Init(ISessionInfo sessionInfo);

        void Clear();
    }
}